// Texture variables.
Texture2D<float> YTexture;
Texture2D<float> UTexture;
Texture2D<float> VTexture;
Texture2D<float> ATexture;

SamplerState YSampler;
SamplerState USampler;
SamplerState VSampler;
SamplerState ASampler;

struct VS_OUT
{
    float4 pos   : SV_Position;
    float2 uv    : TEXCOORD0;
    float4 color : COLOR0;
};

// Constants for the YUV to RGB conversion.
static const float3   ColorBias = {-0.062745, -0.50196, -0.50196};
static float3x3 ColorTransform = { 1.1641444,  1.1641444,  1.1641444,
								  -0.0017889, -0.3914428,  2.0178255,
								   1.5957862, -0.8134821, -0.0012458};

// Constants to convert incoming alpha to display alpha. This conversion assumes that
// the incoming alpha has been copied from the Y channel of a YUV decoded stream.
float AlphaBias = -0.062745;
float AlphaScale = 1.1641444;

// Pixel shader (YUVA->RGBA).
float4 ps_yuva(VS_OUT In) : COLOR
{
	float3 col;
	col.r = YTexture.Sample(YSampler, In.uv);
	col.g = UTexture.Sample(USampler, In.uv);
	col.b = VTexture.Sample(VSampler, In.uv);
	col += ColorBias;

	float4 res;
	res.rgb = mul(col, ColorTransform);
	res.a = (ATexture.Sample(ASampler, In.uv) + AlphaBias) * AlphaScale;

	return res * In.color;
}

float4 main(VS_OUT input) : SV_Target
{
    return ps_yuva(input);
}


